SystemOrganization addCategory: #GODBases!

Object subclass: #GDBClientController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBClientController commentStamp: 'lacm 12/2/2014 13:15' prior: 0!
A GDBClientController is a main class that permits differents transactions in our database.

Before following these instructions, first set up your server in a separate running image by following the instructions in the GDBServerController class. After you have the server running, switch back to this image and do the following.
	
1.-  First, to conect a client with the database run the following:	

			GDBClientController initializeSession.

2.- To create the database, run:

			GDBDatabase createDatabase.
				
	If you want to create only the database for a GODData, run:
	
			GDBData createGODDataCollection.
			
	If you want to create only the database for a GCConference, run:		
			
			GDBConference createConferenceCollection.
			
3.  To reset the database, run the following:	 
	
			GDBDatabase resetAllDatabase.
	
	If you want to reset only the database for a GODData, run:
	
			GDBData resetData			
	
	If you want to reset only the database for a GCConference, run:		
								
			GDBConference resetConferenceData
	
4. Transactions for a GODData object.

			GDBData add:  myGODData.
			GDBData remove:  myGODData.			
			GDBData update:  myOldGODData with: myNewGODData.						
									
			GDBData searchAuthor: 'aPartOfAuthorName'.	
			GDBData searchTitle: 'aPartOfTitleName'.	
			GDBData searchFor: aBlock.
				
			GDBData searchById: anIntegerId.
			GDBData exactSearchByAuthor: 'anAuthorName'.
			GDBData exactSearchByTitle: 'aTitleName'.			

5. Transactions for a GCConference object.

			GDBConference saveConferences:  myConferenceCollection.
			GDBConference loadConferences.			
			
6. To update any change by other client, before any transaction run the following:
	
			GDBClientController refresh.
			
7. To change a client session, run the following:

			GDBClientController refresh: 'nameOfOtherClient'.			

8. To disconnect the client session, run the following:

			GDBClientController release.!

!GDBClientController class methodsFor: 'accessing' stamp: 'lacm 11/20/2014 16:52'!
initializeSession
"This method permits to initialize the client session"
	^GDBMagmaClient session! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/14/2014 16:51'!
refresh
	"This method permits to refresh a client session"
	|aclient|
	aclient:=GDBMagmaClient aClientName. 
	self release.
	GDBMagmaClient aClientName: aclient. 
	^self initializeSession refresh.
! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/14/2014 16:51'!
refresh: aClientName
	"This method permits to refresh a client session, where aclientName is the client name"
	self release.
      GDBMagmaClient aClientName: aClientName.
	^self initializeSession refresh.
! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/20/2014 16:55'!
release
"This method permits to release the client session"
	GDBMagmaClient release.! !

Object subclass: #GDBConference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBConference commentStamp: 'lacm 12/2/2014 13:18' prior: 0!
A GDBConference is a class that contains methods for a GCConference. To more instructions, please see in the GDBClientController.
!

!GDBConference class methodsFor: 'private' stamp: 'lacm 12/1/2014 14:29'!
conferenceCollectionExists
"I verify if the collection that saves a collection of GCCongerence exists in the database."
	|auxDatabaseConference|
	auxDatabaseConference := GDBMagmaClient root keys.
	auxDatabaseConference do: [:e| (e ='gdbconference') ifTrue:[^true].].
	
	^false
	! !

!GDBConference class methodsFor: 'creating' stamp: 'lacm 12/1/2014 15:48'!
createConferenceCollection
"This method creates a collection to store a collection of GCConference objects in the database"
	GDBClientController initializeSession.
	GDBMagmaClient commit: [GDBMagmaClient root at: 'gdbconference' put: OrderedCollection new. ].
! !

!GDBConference class methodsFor: 'private' stamp: 'lacm 11/30/2014 15:11'!
databaseConference
      "new key for the root dictionary"	
	^ GDBMagmaClient root at: 'gdbconference'.! !

!GDBConference class methodsFor: 'getting' stamp: 'lacm 12/1/2014 14:29'!
loadConferences
"This method permits to retrieve the collection of GCConference objects saved in the database"
	|myReader|
	(self conferenceCollectionExists = true) 
		ifTrue: [ (self databaseConference size > 0) 
					ifTrue:[  myReader:=self databaseConference.
							 myReader do:[:e| ^e ].
						].
				].
	^nil! !

!GDBConference class methodsFor: 'resetting' stamp: 'lacm 11/30/2014 15:04'!
resetConferenceData
"This method clears the collection that saves a collection of GCConference objects in the database"
	self createConferenceCollection.
! !

!GDBConference class methodsFor: 'adding' stamp: 'lacm 12/1/2014 21:01'!
saveConferences: aCollectionOfGCConference
	"This method permits to save a collection of GCConference objects in the database"

	self createConferenceCollection.	
      GDBMagmaClient commit: [ 	self databaseConference add:  aCollectionOfGCConference.].	
! !

Object subclass: #GDBData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBData commentStamp: 'lacm 12/2/2014 13:18' prior: 0!
A GDBData is a class that contains methods for a GODData. To more instructions, please see in the GDBClientController.

!

!GDBData class methodsFor: 'adding' stamp: 'lacm 12/1/2014 21:03'!
add: aGODData
	"This method permits to insert a GODData object in the database"
      (aGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (aGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (aGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
	((self godDataExists: aGODData) = true)
		ifTrue:[GODDataAlreadyExistsInTheDatabase signal].

      GDBMagmaClient commit: [self database add:  (self updateId: aGODData).].	


! !

!GDBData class methodsFor: 'creating' stamp: 'lacm 11/30/2014 15:31'!
createGODDataCollection
"This method creates a collection to store GODData objects in the database"
	GDBMagmaClient commit: [
								GDBMagmaClient root at: 'gdbdata' put: (MagmaCollection new		
													addIndex:(MaIntegerIndex attribute: #id);	
													addIndex:(MaSearchStringIndex attribute: #author);		
													addIndex:(MaSearchStringIndex attribute: #title);		
													yourself).
								].
! !

!GDBData class methodsFor: 'private' stamp: 'lacm 11/30/2014 15:34'!
database
      "new key for the root dictionary"
	^ GDBMagmaClient root at: 'gdbdata'.! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 21:54'!
exactSearchByAuthor: aString
	"This method searches GODData objects by Author. This permits an exact search that returns a collection."	
	|aBlock collection|
	(self getAll size > 0) 
		ifTrue:[
			aBlock := [:obj | obj author = aString].
			collection :=  GDBData searchFor: aBlock.
			(collection size > 0) ifTrue:[ ^collection].
		].
	^nil
	! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 21:54'!
exactSearchByTitle: aString
	"This method searches GODData objects by Title. This permits an exact search that returns a collection."	
	|aBlock collection|
	(self getAll size > 0) 
		ifTrue:[
			aBlock := [:obj | obj title = aString].
			collection :=  GDBData searchFor: aBlock.
			(collection size > 0) ifTrue:[ ^collection].
		].
	^nil! !

!GDBData class methodsFor: 'getting' stamp: 'lacm 11/30/2014 15:31'!
getAll   
	"This method gets all element of the database."
	| myReader|	
	myReader := self database.
	^myReader! !

!GDBData class methodsFor: 'private' stamp: 'lacm 12/1/2014 14:27'!
godDataCollectionExists
"I verify if the collection that saves GODData objects exists in the database."
	|auxGODDataCollection|
	auxGODDataCollection := GDBMagmaClient root keys.
	auxGODDataCollection do: [:e| (e ='gdbdata') ifTrue:[^true].].
	
	^false! !

!GDBData class methodsFor: 'adding' stamp: 'lacm 12/1/2014 14:27'!
godDataExists: aGODData
	"This method permits to verify if a GODData object exists in the database"
      |myReader|
	(self database size = 0)
	ifFalse:[ myReader:= self getAll.
			myReader do:[:each| (each=aGODData)
									ifTrue:[^true]
						   ].
		    ].
	^false! !

!GDBData class methodsFor: 'removing' stamp: 'lacm 12/1/2014 21:06'!
remove: aGODData
	"This method permits to delete a GODData object in the database"
	|myReader|
	(self database size = 0) 
		ifFalse: [
				myReader := self getAll. 
    				myReader do: [:e| (e = aGODData ) 
									ifTrue: [GDBMagmaClient commit: [self database remove: e ].
											^true.].
								].
				].
	^false
! !

!GDBData class methodsFor: 'resetting' stamp: 'lacm 11/30/2014 15:32'!
resetData
"This method clears all GODData objects saved in the database"
	self createGODDataCollection.
! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 21:09'!
searchAuthor: aString    
	"This method permits a search by author. This is a non-exact search that returns a collection."	
	| collection myReader temp oCollection|	
	
	temp:= '*',aString,'*'.
	collection:= self getAll.
	myReader:= collection select:[:each | temp match: each author].
	oCollection:= OrderedCollection new.
	(myReader size = 0)	
		ifFalse:[	myReader do: [:each| oCollection add: each.	].
				^oCollection.	
		].
	
	^nil! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 21:10'!
searchById: anInteger
	"This method searches GODData objects by Id and returns a GODData object"
	|aBlock collection|
	(self getAll size > 0) 
		ifTrue:[
			aBlock := [:obj | obj id = anInteger].
			collection :=  GDBData searchFor: aBlock.
			collection do:[:e| ^e.]].
	^nil
	
! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 19:41'!
searchFor: aBlock    
	"This method searches for elements that satisfies aBlock conditions"
	| aBag collection oCollection|	

	collection :=  self getAll.

	aBag := Bag new.
	collection do: [:a| (aBlock value: a) ifTrue: [ aBag add: a.] ].
	oCollection := OrderedCollection new.
	aBag do: [:e| oCollection add: e ].

	^ oCollection! !

!GDBData class methodsFor: 'searching' stamp: 'lacm 12/1/2014 21:11'!
searchTitle: aString    
	"This method permits a search by title. This is a non-exact search that returns a collection."		
	| collection myReader temp oCollection|	
	
	temp:= '*',aString,'*'.
	collection := self getAll .
	myReader:= collection select:[:each| temp match: each title].
	oCollection:= OrderedCollection new.
	(myReader size = 0)	
		ifFalse:[	myReader do: [:each| oCollection add:each.].
				^oCollection.
		].
	
	^nil! !

!GDBData class methodsFor: 'updating' stamp: 'lacm 12/1/2014 21:14'!
update: oldGODData with: newGODData
	"This method permits to update a GODData object in the database.  This returns True if it updates the data, otherwise returns False."
      | auxReader|
      (oldGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (newGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (oldGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (newGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (oldGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
      (newGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].

	(self database size =0)
		ifFalse:[
			auxReader := self getAll.
			auxReader do: [:e| (e = oldGODData)
								ifTrue:[  newGODData id: oldGODData id.
										GDBMagmaClient commit: [
												self database remove: e.
												self database add: newGODData.].	
										^true.				
									  ].
							].
			].

	^ false! !

!GDBData class methodsFor: 'private' stamp: 'lacm 11/30/2014 15:35'!
updateId: aGODData
	"this method updates the GODData id respect to the database"
	|newGODData newId myReader|	
		
	myReader := self database where: [:reader | reader read: #id from: 1].
	(myReader size = 0) 
		ifTrue: [newId :=1];
		ifFalse: [newId:= (myReader detectMax: [:each | each id]) id +1].
	aGODData id: newId.
	newGODData:= aGODData copy.
	
	^newGODData! !

Object subclass: #GDBDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBDatabase commentStamp: 'lacm 12/2/2014 13:20' prior: 0!
A GDBDatabase is class that contains methods to create collections in the repository. To more instructions, please see in the GDBClientController.

!

!GDBDatabase class methodsFor: 'creating' stamp: 'lacm 11/30/2014 15:40'!
createDatabase
"This method creates two collections to store GODData objects and a collection of GCConference objects in the database"
	GDBMagmaClient commit: [
								GDBData createGODDataCollection.
								GDBConference createConferenceCollection.].
	GDBMagmaClient root.
! !

!GDBDatabase class methodsFor: 'resetting' stamp: 'lacm 11/30/2014 15:41'!
resetAllDatabase
"This method clears all data in the database"
	GDBConference resetConferenceData.
	GDBData resetData.
! !

Object subclass: #GDBMagmaClient
	instanceVariableNames: 'session'
	classVariableNames: 'DefaultConnection'
	poolDictionaries: ''
	category: 'GODBases'!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient commentStamp: 'lacm 11/8/2014 13:19' prior: 0!
A GDBMagmaClient is a class that permits to initialize a client session.!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName
	^aClientName! !

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName: aString
	aClientName:= aString! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 9/28/2014 20:53'!
commit: aBlock
	self default session commit: aBlock! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:36'!
default
	^ DefaultConnection ifNil: [
			DefaultConnection := self new connect: aClientName; 
			yourself
			].! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 11/30/2014 14:46'!
initialize
	aClientName:= 'default_user'! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 10/15/2014 12:21'!
release
	(DefaultConnection isNil)
	ifFalse:[
			DefaultConnection session disconnectAndClose.
			DefaultConnection := nil.
		]! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
root
"This method retrieves the connection for access to the session, and then access to the root method of Magma "
	^ self session root! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
session
	^ self default session! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/30/2014 14:44'!
connect
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: 'default_user'.! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:27'!
connect: aString
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: aString.! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 9/28/2014 20:51'!
session
	^ session! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 10/15/2014 12:23'!
session: anObject
	session := anObject! !

Object subclass: #GDBMagmaServer
	instanceVariableNames: 'gdbServer repositoryName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBMagmaServer commentStamp: 'lacm 12/2/2014 13:20' prior: 0!
A GDBMagmaServer is a class that permits start and stop the server.!

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:05'!
createGDBRepository
self isCreated
      ifFalse:
	[MagmaRepositoryController
		create: self repositoryName
		root:  Dictionary new].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 12/1/2014 12:01'!
createGDBRepository: aRepositoryName

	self repositoryName: aRepositoryName.
	self createGDBRepository.! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/12/2014 14:43'!
deleteGDBRepository
	self isCreated      
		ifTrue:	[GDBClientController release.	
				 self stopGDBServer.	
				 MagmaRepositoryController		
						delete: self repositoryName.].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 12/1/2014 11:39'!
deleteGDBRepository: aRepositoryName
	(self isCreated: aRepositoryName)
		ifTrue:	[GDBClientController release.	
				 self stopGDBServer.	
				 MagmaRepositoryController		
						delete: aRepositoryName.].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 12/1/2014 21:15'!
detectRunningMagmaServerConsole
	^MagmaServerConsole allInstances
		detect: [:aServerConsole| aServerConsole isRunning]
		ifNone: [^false]! !

!GDBMagmaServer methodsFor: 'initialize-release' stamp: 'lacm 11/30/2014 14:48'!
initialize	
	repositoryName:= 'default_repository'! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/13/2014 21:23'!
inspectServer
	self detectRunningMagmaServerConsole inspect! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 12/1/2014 11:38'!
isCreated	
"This method verifies if a default repository was created"
	^(FileDirectory forFileName: '') directoryExists: self repositoryName! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 12/1/2014 11:37'!
isCreated: aRepositoryName
"This method verifies if a specific repository was created"
	^(FileDirectory forFileName: '') directoryExists: aRepositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 10/15/2014 12:09'!
port
	^ 51001! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName
	^  repositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName: aRepositoryName
	 repositoryName:= aRepositoryName.! !

!GDBMagmaServer methodsFor: 'connection' stamp: 'lacm 11/3/2014 16:45'!
startGDBServer
	self createGDBRepository.
	self stopGDBServer.
	gdbServer := MagmaServerConsole new
		open: self repositoryName;
		processOn: self port;
		yourself.
	self inspect! !

!GDBMagmaServer methodsFor: 'connection' stamp: 'lacm 12/1/2014 11:58'!
startGDBServer: aRepositoryName
	self repositoryName: aRepositoryName.
	self startGDBServer.! !

!GDBMagmaServer methodsFor: 'connection' stamp: 'lacm 11/5/2014 15:11'!
stopGDBServer
	MagmaServerConsole allInstancesDo:
		[:aServerConsole| aServerConsole shutdown]! !

Object subclass: #GDBServerController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

GDBServerController class
	instanceVariableNames: 'uniqueServerInstance'!

!GDBServerController commentStamp: 'lacm 12/2/2014 13:27' prior: 0!
A GDBControllerServer is a main class that permits differents transactions in the server.

To create the repository and connect the server, run only once the following lines in a workspace:

		GDBServerController createRepository.
		or
		GDBServerController createRepository: 'aRepositoryName'.
		
		and
		
		GDBServerController startServer
		or
		GDBServerController startServer: 'aRepositoryName'

if you only run:
		GDBServerController startServer. 
		or
		GDBServerController startServer: 'aRepositoryName'. 
the repository is created by default.

		
Then, save it as an image server. To close the server, close that image saved, and when you open it again, the server will already be running.

To delete the repository run the following:

		GDBServerController deleteRepository.
		or
	      GDBServerController deleteRepository: 'aRepositoryName'		

these commands stop the server.

To stop the server you can run the following:

		GDBServerController stopServer.
		
To know if the server was started run the following:

		GDBServerController isStarted.!

GDBServerController class
	instanceVariableNames: 'uniqueServerInstance'!

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 12/1/2014 12:04'!
createRepository	
	"This method permits to create a repository by default (i.e., the database)"
	self uniqueServerInstance createGDBRepository.! !

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 12/1/2014 12:03'!
createRepository: aRepositoryName
	"This method permits to create a specific repository"
	self uniqueServerInstance createGDBRepository: aRepositoryName.! !

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 12/1/2014 12:05'!
deleteRepository	
	"This method permits to delete the current repository (i.e., the database)"
	self uniqueServerInstance deleteGDBRepository.! !

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 12/1/2014 11:52'!
deleteRepository: aRepositoryName
	"This method permits to delete a specific repository"
	self uniqueServerInstance deleteGDBRepository: aRepositoryName.! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 12/1/2014 14:25'!
isStarted
	(self uniqueServerInstance detectRunningMagmaServerConsole = false) ifTrue:[^false].
	^true! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 11/11/2014 21:08'!
startServer
	"This method permits to start the server"
	self uniqueServerInstance startGDBServer.! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 12/1/2014 11:53'!
startServer: aRepositoryName
	"This method permits to start the server with a specific repository"
	self uniqueServerInstance startGDBServer: aRepositoryName.! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 11/12/2014 11:08'!
stopServer
	"This method permits to stop the server"
	self uniqueServerInstance stopGDBServer.
	uniqueServerInstance := nil.! !

!GDBServerController class methodsFor: 'accessing' stamp: 'lacm 11/11/2014 21:06'!
uniqueServerInstance
	"This method permits to start the server"
	uniqueServerInstance ifNil: [uniqueServerInstance:= GDBMagmaServer new].
	^uniqueServerInstance! !
GDBMagmaClient initialize!
